//
//  AMImageView.m
//  DeskFun
//
//  Created by Andreas on 28.12.04.
//  Copyright 2004 Andreas Mayer. All rights reserved.
//
//	2005-05-30	Andreas Mayer
//	- -drawRect: invalidates window shadow if rep has alpha channel

#import "AMImageView.h"


float const AMImageMinimumWidth = 32.0;
float const AMImageMinimumHeight = 32.0;


@interface AMImageView (Private)
- (void)sendDisplayNextFrame;
- (void)displayFirstFrame:(id)sender;
- (void)displayNextFrame:(id)sender;
- (NSBitmapImageRep *)animationImageRep;
- (void)setAnimationImageRep:(NSBitmapImageRep *)newAnimationImageRep;
- (NSBitmapImageRep *)bitmapImageRepForAnimation;
- (float)currentFrameDelay;
@end


@implementation AMImageView

// ============================================================
#pragma mark -
#pragma mark ━ initializers ━
// ============================================================

+ (void)initializers
{
	[self exposeBinding:@"image"];
}

- (id)initWithFrame:(NSRect)frame image:(NSImage *)theImage
{
	if (self = [super initWithFrame:frame]) {
		[self setImage:theImage];
		[self setAlpha:1.0];
		[self setAnimationSpeed:1.0];
		[self setImageBorder:AMMakeBorder(0, 0, 0, 0)];
		[self setImageSize:frame.size];
	}
	return self;
}

- (id)initWithFrame:(NSRect)frame
{
	self = [self initWithFrame:frame image:nil];
	return self;
}

- (id)initWithCoder:(NSCoder *)decoder
{
	self = [super initWithCoder:decoder];
	[self setImageBorder:AMMakeBorder(0, 0, 0, 0)];
	[self setImageSize:[self frame].size];
	return self;
}

- (void)dealloc
{
	[image release];
	[super dealloc];
}



// ============================================================
#pragma mark -
#pragma mark ━ accessor methods ━
// ============================================================

- (NSImage *)image
{
	return image; 
}

- (void)setImage:(NSImage *)newImage
{
	//NSLog(@"setImage:%@", newImage);
	if (image != newImage) {
		[newImage retain];
		[image release];
		image = newImage;
		[self setAnimationImageRep:[self bitmapImageRepForAnimation]];
		[self setNeedsDisplay:YES];
	}
}

- (float)alpha
{
	return alpha;
}

- (void)setAlpha:(float)newAlpha
{
	alpha = newAlpha;
}

- (BOOL)canAnimate
{
	return ([self animationImageRep] != nil);
}

- (float)animationSpeed
{
	return animationSpeed;
}

- (void)setAnimationSpeed:(float)newAnimationSpeed
{
	if (animationSpeed != newAnimationSpeed) {
		if (newAnimationSpeed < 0.01) {
			// stop animation and display first frame
			[NSObject cancelPreviousPerformRequestsWithTarget:self];
			[self displayFirstFrame:self];
		} else if (animationSpeed < 0.01) {
			// start animation
			[self sendDisplayNextFrame];
		}
		animationSpeed = newAnimationSpeed;
	}
}

- (NSBitmapImageRep *)animationImageRep
{
	return animationImageRep; 
}

- (void)setAnimationImageRep:(NSBitmapImageRep *)newAnimationImageRep
{
	// do not retain
	animationImageRep = newAnimationImageRep;
}

- (AMBorder)imageBorder
{
	return imageBorder;
}

- (void)setImageBorder:(AMBorder)newImageBorder
{
	imageBorder = newImageBorder;
}


- (NSSize)imageSize
{
	return imageSize;
}

- (void)setImageSize:(NSSize)newImageSize
{
	imageSize = newImageSize;
}


- (NSSize)minSize;
{
	NSSize result;
	AMBorder border = [self imageBorder];
	result.width = AMImageMinimumWidth+border.left+border.right;
	result.height = AMImageMinimumHeight+border.top+border.bottom;
	return result;
}


// ============================================================
#pragma mark -
#pragma mark ━ NSView methods ━
// ============================================================

- (void)drawRect:(NSRect)rect
{
	NSRect sourceRect;
	NSRect destRect;
	sourceRect.origin = NSZeroPoint;
	sourceRect.size = [[self image] size];
	AMBorder border = [self imageBorder];
	destRect = [self bounds];
	//destRect.origin = NSZeroPoint;
	//destRect.size = [self imageSize];
	destRect.origin.x += border.left;
	destRect.origin.y += border.bottom;
	destRect.size.width -= (border.left+border.right);
	destRect.size.height -= (border.top+border.bottom);

	[image drawInRect:destRect fromRect:sourceRect operation:NSCompositeSourceOver fraction:alpha];
#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6
	NSImageRep *rep = [image bestRepresentationForRect:destRect context:nil hints:nil];
#else
	NSImageRep *rep = [image bestRepresentationForDevice:nil];
#endif
	if ([rep hasAlpha]) {
		[[self window] invalidateShadow];
	}
}


// ============================================================
#pragma mark -
#pragma mark ━ private methods ━
// ============================================================

- (NSBitmapImageRep *)bitmapImageRepForAnimation
{
	NSBitmapImageRep *result = nil;
	NSArray *imageReps = [[self image] representations];
	NSEnumerator *enumerator = [imageReps objectEnumerator];
	NSBitmapImageRep *representation;
	while (representation = [enumerator nextObject]) {
		if ([representation isKindOfClass:[NSBitmapImageRep class]]) {
			if ([[representation valueForProperty:NSImageFrameCount] intValue] > 1) {
				result = representation;
				break;
			}
		}
	}
	return result;
}

- (float)currentFrameDelay
{
	float result = [[[self animationImageRep] valueForProperty:NSImageCurrentFrameDuration] floatValue];
	if ([self animationSpeed] >= 0.01) {
		result /= [self animationSpeed];
	}
	result = MAX(1/30, result);
	//NSLog(@"currentFrameDelay: %f", result);
	return result;
}

- (void)sendDisplayNextFrame
{
	// sends displayNextFrame: after current frame duration
	if ([self canAnimate]) {
		[self performSelector:@selector(displayNextFrame:) withObject:self afterDelay:[self currentFrameDelay]];
	}
}

- (void)displayFirstFrame:(id)sender
{
	if ([self canAnimate]) {
		[[self animationImageRep] setProperty:NSImageCurrentFrame withValue:[NSNumber numberWithInt:0]];
		[self setNeedsDisplay:YES];
	}
}

- (void)displayNextFrame:(id)sender
{
	if ([self canAnimate]) {
		
		int frameCount = [[[self animationImageRep] valueForProperty:NSImageFrameCount] intValue];
		int currentFrame = [[[self animationImageRep] valueForProperty:NSImageCurrentFrame] intValue];
		//NSLog(@"current frame: %i", currentFrame);
		currentFrame++;
		if (currentFrame >= frameCount) currentFrame = 0;
		//NSLog(@"next frame: %i", currentFrame);
		[[self animationImageRep] setProperty:NSImageCurrentFrame withValue:[NSNumber numberWithInt:currentFrame]];
		[self setNeedsDisplay:YES];
		[self performSelector:@selector(displayNextFrame:) withObject:self afterDelay:[self currentFrameDelay]];
	}
}


@end
